import argparse
import notdb
import pyonr
import os
from getpass import getpass
from bcrypt import checkpw
from app import create_app

from __init__ import __version__ as v

def main():
   pass

def refresh_data(file:pyonr.Read):
   return file.readfile

def get_password():   
   return getpass('Database password: ').encode('utf-8')

def get_port():
   inp = input('Server PORT (default=5000): ')
   if inp:
      return int(inp)
   return 5000

def is_taken_port(port: int):
   # got this from https://stackoverflow.com/questions/2470971/fast-way-to-test-if-a-port-is-in-use-using-python
   import socket
   with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
      return s.connect_ex(('localhost', port)) == 0

parser = argparse.ArgumentParser('NotDB Viewer', 'notdb_viewer [filename]', f'NotDB Viewer tool v{v}')
parser.add_argument('filename', nargs=1, type=str, help='Run a webapp to view your database data', metavar='filename')

args = parser.parse_args()

if len(args.filename) != 0:
   filename = args.filename[0]
   
   if not os.path.isfile(filename):
      parser.error('Invalid filename/path')
   

   file = pyonr.Read(filename)
   filedata = refresh_data(file)
   db = None
   if filedata.get('__password'):
      _p = get_password()
      if not checkpw(_p, filedata.get('__password')):
         parser.error('Wrong password.')

      db = notdb.NotDBClient(filename, password=_p)
   else:
      db = notdb.NotDBClient(filename)
         
   
   port = get_port()
   if is_taken_port(port):
      parser.error('Used port')

   wa = create_app()

   wa.db = db
   wa.file = file

   wa.run(port=port)


from flask import Flask, render_template_string, request
import pyonr
import notdb
from requests import get
from flask_minify import minify

def refresh_data(file:pyonr.Read) -> dict:
   return file.readfile

def get_class(obj):
   return type(obj)

def get_obj_class_name(obj):
   return type(obj)().__class__.__name__

viewer_html = '''


<!DOCTYPE html>
<html lang="en">
<head>
   <meta charset="UTF-8">
   <meta http-equiv="X-UA-Compatible" content="IE=edge">
   <meta name="viewport" content="width=device-width, initial-scale=1.0">
   
   <!-- Bootstrap -->
   <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.1.3/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-1BmE4kWBq78iYhFldvKuhfTAU6auU8tT94WrHftjDbrCEXSU1oBoqyl2QvZ6jIW3" crossorigin="anonymous">
   <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.1.3/dist/js/bootstrap.bundle.min.js" integrity="sha384-ka7Sk0Gln4gmtz2MlQnikT1wXgYsOg+OMhuP+IlRH9sENBO0LRn5q+8nbTov4+1p" crossorigin="anonymous"></script>

   <!-- JQuery -->
   <script src="https://ajax.googleapis.com/ajax/libs/jquery/3.6.0/jquery.min.js"></script>

   <!-- FontAwesome -->
   <link rel="stylesheet" href="https://pro.fontawesome.com/releases/v5.10.0/css/all.css" integrity="sha384-AYmEC3Yw5cVb3ZcuHtOA93w35dYTsvhLPVnYs9eStHfGJvOvKxVfELGroGkvsg+p" crossorigin="anonymous" />
   
   <title>{{ host }}</title>
   <style>
      :root {
         --dark_bg: #0d1b2a;
         --dark_bg2: #293958;
         --main_dark: #415a77;
         --main_light: #778da9;
         --main_almost_super_light: #bdc0c5;
         --main_super_light: #e0e1dd;
      }

      @keyframes rotate {
         from { transform: rotate(0deg) } 
         to { transform: rotate(360deg) } 
      }
      
      body {
         background-color: var(--dark_bg);
         color: var(--main_super_light)
      }

      div.container {
         margin-top: 10px;
      }

      div.host-container {
         margin-top: 20px;
      }
      
      div.host-container div.host-text {
         background-color: var(--dark_bg2);
         padding: 20px;
         border-radius: 10px;
         user-select: all;
      }

      div.info-container {
         margin-top: 20px;
      }

      div.info-container p {
         margin-bottom: 0.3rem;
      }

      div.info-container p .val {
         background-color: var(--dark_bg2);
         padding: 3px;
         border-radius: 5px;
      }

      div.documents-container, .data-h5 {
         margin-top: 20px
      }

      .docs-title {
         display: flex;
         justify-content: space-between;
         align-items: center;
      }

      .refresh svg {
         width: 25px;
         fill: var(--main_super_light);
         cursor: pointer;
      }
      
      .rotate-ani {
            animation: rotate 1.5s;
            animation-timing-function: cubic-bezier( 0.25, 0.82, 1, 1 ) ;
      }

      div.documents-container div.doc-container {
         background-color: var(--dark_bg2);
         padding: 20px;
         border-radius: 10px;
         margin-bottom: 20px;
      }

      div.documents-container div.doc-container .doc-data {
         display: flex;
         justify-content: space-between;
      }

      div.documents-container div.doc-container .doc-data .type {
         opacity: 80%;
      }

      .no-data {
         margin-top: 100px;
         font-size: 30px;
         display: flex;
         justify-content: center;
      }

      nav {
         display: flex;
         justify-content: space-between;
         align-items: center;
      }

      nav a {
         color: var(--main_super_light);
         transition: 0.3s;
         text-decoration: none;
      }

      nav a:hover {
         color: var(--main_almost_super_light);
      }

      nav .discord {
         font-size: 20px;
      }
   </style>
</head>
<body>
   <div class="container">
      <nav>
         <h2>NotDB Viewer</h2>
         <div class="d">
            Join our official 
            <a href="https://discord.gg/Az8McWNAcg" target="_blank">
              <i class="fab fa-discord discord"></i>
              Discord
            </a>
         </div>
      </nav>
      <div class="host-container">
         <h5>Host:</h4>
         <div class="host-text">
            {{ host }}
         </div>
      </div>
      <div class="info-container" id="info-c">
         {% for text, val in db_info.items() %}
            <div class="info">
               <p>
                  {{ text }}: 
                  <b class="val">{{ val }}</b>
               </p>
            </div>
         {% endfor %}
      </div>
      <div class="docs-title">
         <h5 class="data-h5">Data:</h4>
            <div class="refresh" id="refresh" title="Keyboard shortcut: R" onclick="ani();refreshDocuments()">
               <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><path d="M468.9 32.11c13.87 0 27.18 10.77 27.18 27.04v145.9c0 10.59-8.584 19.17-19.17 19.17h-145.7c-16.28 0-27.06-13.32-27.06-27.2c0-6.634 2.461-13.4 7.96-18.9l45.12-45.14c-28.22-23.14-63.85-36.64-101.3-36.64c-88.09 0-159.8 71.69-159.8 159.8S167.8 415.9 255.9 415.9c73.14 0 89.44-38.31 115.1-38.31c18.48 0 31.97 15.04 31.97 31.96c0 35.04-81.59 70.41-147 70.41c-123.4 0-223.9-100.5-223.9-223.9S132.6 32.44 256 32.44c54.6 0 106.2 20.39 146.4 55.26l47.6-47.63C455.5 34.57 462.3 32.11 468.9 32.11z"/></svg>
            </div>
         </div>
         <div class="documents-container" id="doc-c">
            {% for document in documents %}
               <div class="doc-container">
                  {% for key, val in document.items() %}
                  <div class="doc-data">
                        <div class="keyval">
                           {{ key }}: <b class="doc-val">{{ val }}</b>
                        </div>
                        <div class="type" title="{{ get_class(val) }}">
                           type: <b>{{ get_obj_class_name(val) }}</b>
                        </div>
                     </div>
                  {% endfor %}
               </div>
            {% endfor %}
            {% if not documents %}
            <h5 class="no-data">There is no data in your Database :(</h5>
            {% endif %}
         </div>
   </div>

   <script>
      function refreshDocuments() {
         $('#doc-c').load(document.URL + ' #doc-c')
         $('#info-c').load(document.URL + ' #info-c')
      }

      function ani() {
         var ele = document.getElementById('refresh')
         if(ele.classList.contains('rotate-ani')) {
            ele.classList.remove('rotate-ani')
         } else {
            ele.classList.add('rotate-ani')
            setTimeout(() => {
               ele.classList.remove('rotate-ani')
            }, 1500)
         }
      }

      document.addEventListener('keyup', (e) => {
         if(e.key == 'r') {
            ani()
            refreshDocuments()
         }
      })
   </script>
</body>
</html>


'''

def create_app():

   app = Flask(__name__)

   minify(app, html=True, js=True, cssless=True, fail_safe=True)

   @app.route('/')
   def index():
      db = app.db
      file = app.file

      data = refresh_data(file)
      db_info = {}

      db_info['Secured with password'] = True if data.get('__password') else False
      db_info['documents'] = db.documents

      return render_template_string(viewer_html,
                              documents=db.get({}),
                              db_info=db_info,
                              host=db.host,
                              get_obj_class_name=get_obj_class_name,
                              get_class=get_class,
                              f=file)

   return app


from notdb import NotDBClient
from time import time

client = NotDBClient('t.ndb', '123')

# while True:
   # val = input('> ')

   # if val == 'print':
   #    s = time()
   #    print(client.get({}))
   #    print(time() - s)
   # elif val == 'clear':
   #    s = time()
   #    client.removeMany({})
   #    print(time() - s)
   # else:
   #    s = time()
   #    client.appendOne({'username': val})
   #    print(time() - s)


# isLoggedIn = False
# username = None

# while True:
#    cmd = input('> ').lower().strip()

#    if cmd == 'whoami':
#       if not isLoggedIn:
#          print('Login/Register first')
#       else:
#          print(username)
   
#    elif cmd == 'register':
#       if isLoggedIn:
#          print('You are logged in')
#       else:
#          username = input('username: ').lower()
#          password = input('password: ')
#          client.appendOne({
#             'username': username,
#             'password': password
#          })
#          print('You have registered successfully.')
   
#    elif cmd == 'login':
#       username = input('username: ').lower()
#       password = input('password: ')

#       print({'username': 'nawaf', 'password': '123'})
#       print({'username': username, 'password': password})
#       print({'username': 'nawaf', 'password': '123'} == {'username': username, 'password': password})

#       user = client.getOne({
#          'username': username
#       })

#       print(user)

#       if not user:
#          print('Invalid username or password')
#       else:
#          isLoggedIn = True
#          username = user['username']
      
#    elif cmd == 'logout':
#       isLoggedIn = False
#       username = None


from setuptools import setup, find_packages
from notdb_viewer import __version__ as v

# README.md
with open('README.md', 'r', encoding='utf-8') as readme_file:
    readme = readme_file.read()

packages = [
    *find_packages()
]

setup(
    name='notdb_viewer',
    packages=packages,
    install_requires=[
        'pyonr',
        'bcrypt',
        'flask',
        'flask_minify'
    ],
    version=v,
    description='Viewer for NotDB Databases',
    author='Nawaf Alqari',
    author_email='nawafalqari13@gmail.com',
    keywords=['notdb', 'db', 'database', 'notdatabsae', 'simple database'],
    long_description=readme,
    long_description_content_type='text/markdown',
    entry_points={
        'console_scripts': ['notdb_viewer=notdb_viewer.__main__:main']
    },
    license='MIT',
    zip_safe=False,
    url='https://github.com/nawafalqari/NotDB_Viewer/',
    project_urls={
        'Documentation': 'https://github.com/nawafalqari/NotDB_Viewer#readme',
        'Bug Tracker': 'https://github.com/nawafalqari/NotDB_Viewer/issues',
        'Source Code': 'https://github.com/nawafalqari/NotDB_Viewer/',
        'Discord': 'https://discord.gg/cpvynqk4XT'
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)